/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.EditGroupViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.SelectionWidget");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");

var CommonMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var CoreAdminMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Displays the Edit Group view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.EditGroupViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/EditGroupViewer.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
		this._args = args;		
	},
	
	postCreate: function() {
		Util.removeChildren(this.pageTitle);
		this.pageTitle.appendChild(document.createTextNode(CoreAdminMessages.EDITGROUP_VIEWER_PAGE_TITLE));
		
		Util.removeChildren(this.deleteButton);
		this.deleteButton.appendChild(document.createTextNode(CommonMessages.DELETE_BUTTON_LABEL));
		
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		Util.removeChildren(this.name);
		this.name.appendChild(document.createTextNode(CoreAdminMessages.EDITGROUP_VIEWER_GROUP_NAME_LABEL));
		
		this.groupName.value = this._args.groupName;
				
		var self = this;
		
		this._members = [];
		this._users = [];
		this._selectionWidget = null;
		this._msgFormatter = new MessageFormatter(this.messageArea);

		dojo.connect(this.deleteButton, "onclick",
			function(e) {
				e.preventDefault();
				self._deleteGroup();
			}
		);
		
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._updateGroup();
			}
		);
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				self._controller.manageUsers();
			}
		);
			
		var responseHandler = new ServiceResponseHandler(this, "_listMembers", "_getGroupError");
		UserClient.getGroup(responseHandler, {name: this.groupName.value});
	},
	
	_listMembers: function(group) {	
		var members = group.members;
		if (typeof members != "undefined" && members != null && members.length > 0) {
			members.sort();
			this._members = members;
		}
		else {
			this._members = [];
		}
		this._getAllUsers();
	},
	
	_getGroupError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_GROUPS, error);
	},
	
	_getAllUsers: function() {
		var responseHandler = new ServiceResponseHandler(this, "_listUsers", "_getAllUsersError");
		UserClient.getAllUsers(responseHandler);		
	},
	
	_listUsers: function(users) {
		this._users = [];
		if (users != null && users.length > 0) {
			var members = this._members;
			for (var i = 0; i < users.length; i++) {
				var user = users[i];
				if (user != this.groupName.value && (members == null || this._isMember(members, user) == false)) {
					this._users.push(user);
				}
			}
		}
		this._createSelectionWidget();
	},
		
	_getAllUsersError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_USERS, error);
	},
	
	_createSelectionWidget: function() {
		var self = this;
		this._msgFormatter.displayNoMessage();			
		
		var availableOptions = [];
		var selectedOptions = [];

		if (this._users != null) {
			for (var i = 0; i < this._users.length; i++ ) {
				var user = this._users[i];
				availableOptions.push({value: user.name, text: user.name});
			}
		}
					
		if (this._members != null) {
			for (var i = 0; i < this._members.length; i++ ) {
				var user = this._members[i];
				selectedOptions.push({value: user.name, text: user.name});
			}	
		}
		
		/*
		this._selectionWidget = new com.ibm.rational.report.web.core.app.internal.common.SelectionWidget(
			{availableOptions: availableOptions, selectedOptions: selectedOptions,
			 availableItemsTitle: "Available Users:", selectedItemsTitle: "Group Members:"});
		*/
		this._selectionWidget = new com.ibm.rational.report.web.core.app.internal.common.SelectionWidget(
			{availableOptions: availableOptions, selectedOptions: selectedOptions,
			 availableItemsTitle: CoreAdminMessages.EDITGROUP_VIEWER_AVAILABLE_USERS_LABEL, selectedItemsTitle: CoreAdminMessages.EDITGROUP_VIEWER_GROUP_MEMBERS_LABEL});
		dojo.connect(this._selectionWidget.availableControl, "onchange",
			function(e){
				self.saveButton.disabled = false;
			}
		);
		dojo.connect(this._selectionWidget.selectedControl, "onchange",
			function(e){
				self.saveButton.disabled = false;
			}
		);
		this.userGroupSelections.appendChild(this._selectionWidget.domNode);
	},
		
	_updateGroup: function() {		
		var self = this;
		var groupName = this.groupName.value;
		
		var handler = {
			_success: function() {
				var msg = dojo.string.substitute(CoreAdminMessages.SUCCESSFULLY_UPDATED_GROUP, [groupName]);
				self._msgFormatter.displaySuccessMessage(msg);
				self._controller.manageUsers();
			},
			_error: function(error) {
				var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_GROUP, [groupName]);
				self._msgFormatter.displayErrorMessage(msg, error);
			}
		};
		var responseHandler = new ServiceResponseHandler(handler, "_success", "_error");
		var selections = this._selectionWidget.getAllSelected();
		var memberNames = [];
		for (var i in selections) {
			memberNames.push(selections[i].value);
		}
		var args = {
			name: groupName,
			memberNames: memberNames
		};
		UserClient.postUpdateGroup(responseHandler, args);		
	},
	
	_deleteGroup: function() {		
		var self = this;
		var groupName = this.groupName.value;
		var handler = {
			_success: function() {
				var msg = dojo.string.substitute(CoreAdminMessages.SUCCESSFULLY_DELETED_GROUP, [groupName]);
				self._msgFormatter.displaySuccessMessage(msg);
				self._controller.manageUsers();
			},
			_error: function(error) {
				var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_DELETE_GROUP, [groupName]);
				self._msgFormatter.displayErrorMessage(msg, error);
			}
		};
		var responseHandler = new ServiceResponseHandler(handler, "_success", "_error");
		UserClient.postDeleteGroup(responseHandler, {name: groupName});			
	},
	
	_isMember: function(members, user) {
		for (var i = 0; i < members.length; i++) {
			var member = members[i];
			if (member.name == user.name) {
				return true;
			}
		}
		return false;
	}
	
});

})();